package com.huimv.apiservice.service.impl;


import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.apiservice.dao.PigDao;
import com.huimv.apiservice.dao.YearPigHistoryImgDao;
import com.huimv.apiservice.entity.YearPigBaseEntity;
import com.huimv.apiservice.dao.*;
import com.huimv.apiservice.entity.*;
import com.huimv.apiservice.entity.vo.*;
import com.huimv.apiservice.service.PigService;
import com.huimv.common.exception.RRException;
import com.huimv.common.utils.PageUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 * 业务层
 * </p>
 *
 * @author yinhao
 * @date 2021/5/8 17:22
 */
@Service
public class PigServiceImpl extends ServiceImpl<PigDao, YearPigBaseEntity> implements PigService {

    @Autowired
    private EartagDataDao eartagDataDao;

    @Autowired
    private PeriodDao periodDao;

    @Autowired
    private IndoorEnvironmentDao indoorEnvironmentDao;

    @Autowired
    private OutdoorEnvironmentDao outdoorEnvironmentDao;

    @Autowired
    private YearPigHistoryImgDao yearPigHistoryImgDao;

    @Autowired
    private HealthMedicalRecordDao healthMedicalRecordDao;

    @Autowired
    private FeedRecordDao feedRecordDao;

    @Override
    public PigHealthStatusVo getPigInfoByEarTagNo(String pigEarTagNo) {

        //耳标检查
        checkPigEarTagNo(pigEarTagNo);

        //当前时间节点
        DateTime currentTime = DateUtil.date();
        //一天前该时间节点的时间
        DateTime oneDayBefore = DateUtil.offsetDay(currentTime, -1);

//        String lastGetDataTime = (String) redisTemplate.opsForHash().get("offset_record", "getPigInfo");
//        redisTemplate.opsForHash().put("offset_record", "getPigInfo",currentTime.toString());

        //健康状态
        PigHealthStatusVo pigHealthStatusVo = new PigHealthStatusVo();
        boolean health = baseMapper.selectHealthStatus(pigEarTagNo);
        pigHealthStatusVo.setHealthStatus(health ? "健康" : "不健康");

        //公用时间 -> 当前日期
        String today = DateUtil.format(currentTime, "yyyy-MM-dd");

        LambdaQueryWrapper<EartagDataEntity> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.eq(EartagDataEntity::getEartagId, pigEarTagNo)
                .ge(EartagDataEntity::getCreateTime, oneDayBefore);

        //查询耳标上传数据
        List<EartagDataEntity> eartagDataEntityList = eartagDataDao.selectList(lambdaQueryWrapper);
        List<AnimalHeatVo> animalHeatVoList = new ArrayList<>();
        List<SportTimeVo> sportTimeVoList = new ArrayList<>();
        List<SleepStatusVo> sleepStatusVoList = new ArrayList<>();
        if (CollUtil.isNotEmpty(eartagDataEntityList)) {
            for (EartagDataEntity eartagDataEntity : eartagDataEntityList) {
                AnimalHeatVo animalHeatVo = new AnimalHeatVo();
                Date createTime = eartagDataEntity.getCreateTime();
                animalHeatVo.setCollectTime(createTime);
                animalHeatVo.setTemperature(eartagDataEntity.getEartagTemp());
                animalHeatVoList.add(animalHeatVo);

                SportTimeVo sportTimeVo = new SportTimeVo();
                sportTimeVo.setCollectTime(createTime);
                sportTimeVo.setSportCount(eartagDataEntity.getCountSport());
                sportTimeVoList.add(sportTimeVo);

                if (eartagDataEntity.getSleepGap() == 1) {
                    SleepStatusVo sleepStatusVo = new SleepStatusVo();
                    sleepStatusVo.setSleepStartTime(eartagDataEntity.getSleepBegin());
                    sleepStatusVo.setDuration(eartagDataEntity.getRemark());
                    sleepStatusVoList.add(sleepStatusVo);
                }
            }
        }
        //温度状态
        pigHealthStatusVo.setTemperature(animalHeatVoList);
        //运动状态
        pigHealthStatusVo.setSportStatus(sportTimeVoList);
        //睡眠状态
        pigHealthStatusVo.setSleepStatus(sleepStatusVoList);

        //采食量
        LambdaQueryWrapper<FeedRecordEntity> feedRecordQuery = Wrappers.lambdaQuery();
        feedRecordQuery.eq(FeedRecordEntity::getPmsPig__earNumber, pigEarTagNo)
                .ge(FeedRecordEntity::getStartTime, oneDayBefore);

        List<FeedRecordVo> feedRecords = feedRecordDao.queryFeedRecords(feedRecordQuery);
        pigHealthStatusVo.setFeedingStatus(feedRecords);

        return pigHealthStatusVo;
    }


    @Override
    public PigImageVo getImageByEarTagNo(String pigEarTagNo) {

        checkPigEarTagNo(pigEarTagNo);

        PigImageVo pigImageVo = new PigImageVo();

        pigImageVo.setPigEarTagNo(pigEarTagNo);
        List<String> images = yearPigHistoryImgDao.getLatestImgByPigEarTagNo(pigEarTagNo);
        if (CollectionUtil.isNotEmpty(images)) {
            //String encodeImg = Base64.encode(img);
            pigImageVo.setImages(images);
        }
        return pigImageVo;
    }


    @Override
    public PageUtils getPageByBreedAndFence(String breedName, Integer currentPage, Integer pageSize, String period) {

        Integer count = baseMapper.selectCount(Wrappers.<YearPigBaseEntity>lambdaQuery().eq(YearPigBaseEntity::getBreed, breedName).last("limit 1"));
        if (count <= 0) {
            throw new RRException("品种名称不存在,请检查!", 1001);
        }

        Integer periodId = null;
        if (StringUtils.isNotEmpty(period)) {
            periodId = periodDao.selectIdByPeriodNumber(period);
            if (periodId == null) {
                throw new RRException("栏期编号不存在,请检查!", 1001);
            }
        }


        IPage<PigBaseVo> page = new Page<>(currentPage, pageSize);
        IPage<PigBaseVo> pigBaseVoList = baseMapper.selectPagePigBaseVoByBreedName(page, breedName, periodId);

        return new PageUtils(pigBaseVoList);
    }

    @Override
    public EnvVo getIndoorEnv(String pigEarTagNo) {

        String today = DateUtil.format(DateUtil.date(), "yyyy-MM-dd");

        LambdaQueryWrapper<YearPigBaseEntity> pigBaseLambdaQuery = Wrappers.lambdaQuery();
        pigBaseLambdaQuery.eq(YearPigBaseEntity::getEartag, pigEarTagNo);
        YearPigBaseEntity yearPigBaseEntity = baseMapper.selectOne(pigBaseLambdaQuery);
        if (yearPigBaseEntity == null) {
            throw new RRException("耳标信息不存在,请检查!", 1001);
        }

        Integer pigstyId = yearPigBaseEntity.getPigstyId();
        if (pigstyId == null) {
            throw new RRException("未查询到对应猪舍,请联系管理员!", 1001);
        }

        Integer farmId = yearPigBaseEntity.getFarmId();
        if (farmId == null) {
            throw new RRException("未查询到对应牧场,请联系管理员!", 1001);
        }

        EnvVo envVo = new EnvVo();
        envVo.setPigEarTagNo(pigEarTagNo);
        envVo.setDate(today);

        IndoorEnvVo indoorEnvVo = new IndoorEnvVo();

        List<ValueVo> temperatureVoList = selectIndoorEnvByType(pigstyId, ((short) 1));
        indoorEnvVo.setTemperatureList(temperatureVoList);
        List<ValueVo> humidityVoList = selectIndoorEnvByType(pigstyId, ((short) 2));
        indoorEnvVo.setHumidityList(humidityVoList);
        List<ValueVo> gasVoList = selectIndoorEnvByType(pigstyId, ((short) 3));
        indoorEnvVo.setGasList(gasVoList);
        envVo.setIndoorEnv(indoorEnvVo);

        LambdaQueryWrapper<OutdoorEnvironmentEntity> outdoorEnvLambdaQuery = Wrappers.lambdaQuery();
        outdoorEnvLambdaQuery.eq(OutdoorEnvironmentEntity::getFarmId, farmId)
                .eq(OutdoorEnvironmentEntity::getDate, today)
                .le(OutdoorEnvironmentEntity::getCollectTime, new Date())
                .eq(OutdoorEnvironmentEntity::getDeleted, 0)
                .orderByDesc(OutdoorEnvironmentEntity::getId);
        List<OutdoorEnvVo> outdoorEnvVoList = outdoorEnvironmentDao.select(outdoorEnvLambdaQuery);
        envVo.setOutdoorEnvList(outdoorEnvVoList);

        return envVo;
    }

    private List<ValueVo> selectIndoorEnvByType(Integer pigstyId, short type) {

        LambdaQueryWrapper<IndoorEnvironmentEntity> indoorEnvLambdaQuery = Wrappers.lambdaQuery();
        indoorEnvLambdaQuery.eq(IndoorEnvironmentEntity::getPigstyId, pigstyId)
                .eq(IndoorEnvironmentEntity::getType, type)
                .ge(IndoorEnvironmentEntity::getCollectTime, DateUtil.beginOfDay(new Date()))
                .le(IndoorEnvironmentEntity::getCollectTime, new Date())
                .eq(IndoorEnvironmentEntity::getDeleted, 0)
                .orderByDesc(IndoorEnvironmentEntity::getId);

        return indoorEnvironmentDao.select(indoorEnvLambdaQuery);
    }

    @Override
    public void adopt(String pigEarTagNo) {

        YearPigBaseEntity yearPigBaseEntity = getYearPigBaseEntity(pigEarTagNo);
        if (yearPigBaseEntity.getFosterStatus()) {
            throw new RRException("本猪已被其他人认养!", 1001);
        }
        LambdaUpdateWrapper<YearPigBaseEntity> pigBaseLambdaUpdate = Wrappers.lambdaUpdate();
        pigBaseLambdaUpdate.eq(YearPigBaseEntity::getEartag, pigEarTagNo)
                .set(YearPigBaseEntity::getFosterStatus, true);

        update(pigBaseLambdaUpdate);
    }

    @Override
    public void outFence(String pigEarTagNo, Integer status) {

        checkPigEarTagNo(pigEarTagNo);

        LambdaQueryWrapper<YearPigBaseEntity> pigBaseLambdaQuery = Wrappers.lambdaQuery();
        pigBaseLambdaQuery.eq(YearPigBaseEntity::getEartag, pigEarTagNo);
        YearPigBaseEntity yearPigBaseEntity = baseMapper.selectOne(pigBaseLambdaQuery);
        Integer outFenceStatus = yearPigBaseEntity.getOutFenceStatus();
        if (outFenceStatus != null) {
            if (outFenceStatus.equals(1) || outFenceStatus.equals(2) || outFenceStatus.equals(3)) {
                throw new RRException("该猪已出栏,不可重复出栏!", 1001);
            }
        }

        LambdaUpdateWrapper<YearPigBaseEntity> pigBaseLambdaUpdate = Wrappers.lambdaUpdate();
        pigBaseLambdaUpdate.eq(YearPigBaseEntity::getEartag, pigEarTagNo)
                .set(YearPigBaseEntity::getDeleted, System.currentTimeMillis() / 1000)
                .set(YearPigBaseEntity::getFosterStatus, false)
                .set(YearPigBaseEntity::getOutFenceTime, new Date())
                .set(YearPigBaseEntity::getOutFenceStatus, status);

        update(pigBaseLambdaUpdate);

    }

    @Override
    public void abandonAdopt(String pigEarTagNo) {

        YearPigBaseEntity yearPigBaseEntity = getYearPigBaseEntity(pigEarTagNo);

        if ( !yearPigBaseEntity.getFosterStatus() || yearPigBaseEntity.getOutFenceStatus() != 0) {
            throw new RRException("操作失败,该小猪目前未被认养或已经出栏!", 1001);
        }

        LambdaUpdateWrapper<YearPigBaseEntity> pigBaseLambdaUpdate = Wrappers.lambdaUpdate();
        pigBaseLambdaUpdate.eq(YearPigBaseEntity::getEartag, pigEarTagNo)
                .set(YearPigBaseEntity::getFosterStatus, false);
        update(pigBaseLambdaUpdate);
    }

    @Override
    public List<HealthMedicalRecordEntity> getAbnormalStatus(String pigEarTagNo) {

        checkPigEarTagNo(pigEarTagNo);

        LambdaQueryWrapper<HealthMedicalRecordEntity> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(HealthMedicalRecordEntity::getPigId, pigEarTagNo);
        return healthMedicalRecordDao.selectList(wrapper);
    }

    /**
     * 公用检查耳标方法
     *
     * @param pigEarTagNo 耳标号
     */
    private void checkPigEarTagNo(String pigEarTagNo) {

        Integer count = baseMapper.selectCount(
                Wrappers.<YearPigBaseEntity>lambdaQuery()
                        .eq(YearPigBaseEntity::getEartag, pigEarTagNo)
                        .eq(YearPigBaseEntity::getDeleted, 0)
                        .last("limit 1"));
        if (count <= 0) {
            throw new RRException("耳标信息不存在,请检查!", 1001);
        }
    }

    /**
     * 公用获取小猪信息方法
     *
     * @param pigEarTagNo
     * @return
     */
    private YearPigBaseEntity getYearPigBaseEntity(String pigEarTagNo) {
        LambdaQueryWrapper<YearPigBaseEntity> eq = Wrappers.<YearPigBaseEntity>lambdaQuery().eq(YearPigBaseEntity::getEartag, pigEarTagNo);
        YearPigBaseEntity yearPigBaseEntity = baseMapper.selectOne(eq);
        if (yearPigBaseEntity == null) {
            throw new RRException("耳标信息不存在,请检查!", 1001);
        }
        return yearPigBaseEntity;
    }


}
