package com.huimv.apiservice.task;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.huimv.apiservice.dao.FeedRecordDao;
import com.huimv.apiservice.entity.FeedRecordEntity;
import com.huimv.common.exception.RRException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 数据采集定时任务
 * </p>
 *
 * @author yinhao
 * @date 2021/7/21 11:01
 */
@Configuration
@EnableScheduling
//@EnableAsync
public class DataCollectTask {

    @Autowired
    private RestTemplate restTemplate;

    @Value("${runnong.host}")
    private String host;
    @Value("${runnong.login}")
    private String loginUrl;
    @Value("${runnong.feed-record}")
    private String feedRecordUrl;
    @Value("${runnong.xiuzhou.nick-name}")
    private String XzNickName;
    @Value("${runnong.xiuzhou.password}")
    private String XzPassword;

    @Autowired
    private FeedRecordDao feedRecordDao;


    /**
     * 润农测定站读取每天读取4次饲喂记录数据
     */
    @Scheduled(cron = "0 0 0/6 * * ? ")
    void getFeedRecords() {

        //当前时间
        DateTime currentTime = DateUtil.date();

        //组装请求头
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.MULTIPART_FORM_DATA);

        //form表单参数
        MultiValueMap<String, Object> multiValueMap = new LinkedMultiValueMap<>();
        multiValueMap.add("nickName", XzNickName);
        multiValueMap.add("password", XzPassword);
        //构建请求对象
        HttpEntity<MultiValueMap<String, Object>> request = new HttpEntity<>(multiValueMap, headers);
        System.out.println(request.getBody());
        System.out.println(request.getHeaders());


        try {
            //发送获取sessionId的请求
            ResponseEntity<Map> responseEntity = restTemplate.postForEntity(host + loginUrl, request, Map.class);

            //取出响应中的sessionId
            Map body = responseEntity.getBody();
            System.out.println(body);
            if (body == null) {
                throw new RRException("获取登录信息异常!", 1001);
            }
            String sessionId = (String) ((Map) body.get("content")).get("sessionId");
            System.out.println(sessionId);

            //放入获取feedRecord的请求头当中
            headers.set("Cookie", "JSESSIONID=" + sessionId);
            //复用表单,清空条件
            multiValueMap.clear();
            //添加页码
            multiValueMap.add("page", String.valueOf(1));
            //添加行数
            multiValueMap.add("rows", String.valueOf(10000));
            //添加开始时间(获取当天)
            String startTime = null;
            if (currentTime.hour(true) == 0) {
                startTime = DateUtil.format(DateUtil.offsetDay(currentTime, -1), "yyyy-MM-dd");
            } else {
                startTime = DateUtil.format(currentTime, "yyyy-MM-dd");
            }
            multiValueMap.add("dateStart", startTime);
            //添加过滤条件(isValid=true)
            multiValueMap.add("isValid", true);

            //构建请求对象
            request = new HttpEntity<>(multiValueMap, headers);

            //发送获取数据的POST请求
            ResponseEntity<JSONObject> feedRecordResponseEntity = restTemplate.postForEntity(host + feedRecordUrl, request, JSONObject.class);
            //解析响应数据,存入数据库
            JSONObject feedRecordJson = feedRecordResponseEntity.getBody();
            if (feedRecordJson != null) {
                JSONArray rows = feedRecordJson.getJSONArray("rows");
                List<FeedRecordEntity> feedRecordEntities = rows.toJavaList(FeedRecordEntity.class);
                feedRecordDao.batchInsertData(feedRecordEntities);
            }
        } catch (Exception e) {
            throw new RRException(e.getMessage(), 1001, e);
        }

    }


}
