 package com.huimv.management.controller;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import com.huimv.management.entity.SysUserEntity;
import com.huimv.management.util.GetUserUtil;
import io.swagger.annotations.Api;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.huimv.management.entity.CameraInfoEntity;
import com.huimv.management.service.CameraInfoService;
import com.huimv.common.utils.PageUtils;
import com.huimv.common.utils.R;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


 /**
 * 
 *
 * @author yinhao
 * @email yinhao@163.com
 * @date 2021-06-24 09:55:37
 */
@RestController
@RequestMapping("management/camerainfo")
@Api(tags ="摄像头管理")
public class CameraInfoController {
    @Autowired
    private CameraInfoService cameraInfoService;

    /**
     * 列表
     */
    @RequestMapping("/list")
    public R list(@RequestParam Map<String, Object> params,HttpServletRequest request){
        SysUserEntity user = GetUserUtil.getUser(request);
        if (user == null){
            return R.error("获取用户信息异常");
        }
        PageUtils page = cameraInfoService.queryPage(params,user.getUserId());

        return R.ok().put("page", page);
    }


    /**
     * 信息
     */
    @RequestMapping("/info/{id}")
    public R info(@PathVariable("id") Integer id){
		CameraInfoEntity cameraInfo = cameraInfoService.getById(id);

        return R.ok().put("cameraInfo", cameraInfo);
    }

    /**
     * 保存
     */
    @RequestMapping("/save")
    public R save(@RequestBody CameraInfoEntity cameraInfo){
		cameraInfoService.save(cameraInfo);

        return R.ok();
    }

    /**
     * 修改
     */
    @RequestMapping("/update")
    public R update(@RequestBody CameraInfoEntity cameraInfo){
		cameraInfoService.updateById(cameraInfo);

        return R.ok();
    }

    /**
     * 删除
     */
    @RequestMapping("/delete")
    public R delete(@RequestBody Integer[] ids){
		cameraInfoService.removeByIds(Arrays.asList(ids));

        return R.ok();
    }


    @RequestMapping("/getCameraDetails")
    private R getCameraDetails(Integer cameraId){
        if (cameraId == null ){
            return R.error("请选择摄像头");
        }
        List list =  cameraInfoService.getCameraDetails(cameraId);
        return R.ok().put("data",list);
    }

    @RequestMapping("/getByAreaId")
    private R getByAreaId(Integer areaId,HttpServletRequest request){
        SysUserEntity user = GetUserUtil.getUser(request);
        if (user == null){
            return R.error("获取用户信息异常");
        }
        List list = cameraInfoService.getByAreaId(areaId,user.getUserId());
        return R.ok().put("data",list);
    }



    //查询关注列表
    @RequestMapping("/findAttention")
    public R findAttention(Integer farmId, HttpServletRequest request){
        SysUserEntity user = GetUserUtil.getUser(request);
        if (user == null){
            return R.error("获取用户信息异常");
        }
        List list = cameraInfoService.findAttention(farmId,user.getUserId());

        return R.ok().put("data", list);
    }

}
