package com.huimv.management.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.common.utils.PageUtils;
import com.huimv.common.utils.Query;

import com.huimv.management.dao.EartagDataDao;
import com.huimv.management.entity.EartagDataEntity;
import com.huimv.management.service.EartagDataService;


@Service("eartagDataService")
public class EartagDataServiceImpl extends ServiceImpl<EartagDataDao, EartagDataEntity> implements EartagDataService {




    @Autowired
    private EartagDataDao eartagDateDao;


    @Override
    public PageUtils queryPage(Map<String, Object> params) {
        IPage<EartagDataEntity> page = this.page(
                new Query<EartagDataEntity>().getPage(params),
                new QueryWrapper<EartagDataEntity>()
        );

        return new PageUtils(page);
    }


    @Override
    public Map<String,List> queryPageBytimeAndEartag(Map<String, Object> params) throws ParseException {
        //从请求头中找参数  1.耳标ID  2.starttime  3.end_time
        //构建条件查询wrapper
        //耳标
         String eartag = (String) params.get("eartag");
        String starttime = (String) params.get("startTime");
        String endtime = (String) params.get("endTime");
        DateFormat fmt = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Map<String,List> map1 = new HashMap();
        //格式化传入的时间
            Date  startTime = fmt.parse(starttime);
            Date   endTime = fmt.parse(endtime);


        System.out.println("yangdi"+starttime+endtime);



          LambdaQueryWrapper<EartagDataEntity> lambdaQuery = Wrappers.lambdaQuery();
          lambdaQuery.eq(EartagDataEntity::getEartagId,eartag);
           //过滤时间
          lambdaQuery.between(EartagDataEntity::getCreateTime,startTime, endTime);
          List<EartagDataEntity> eartagDateEntities = baseMapper.selectList(lambdaQuery);

        List<String> timeList = new ArrayList<>();
        List<Double>  tempList = new ArrayList<>();
        List<Integer>  sportsList = new ArrayList<>();
        List<Integer> electricList = new ArrayList<>();
        List<Double> environmenttempList = new ArrayList<>();
        List<Integer> sleeptime = new ArrayList<>();

        for (EartagDataEntity data : eartagDateEntities) {
            timeList.add(fmt.format(data.getCreateTime()));
            tempList.add(data.getEartagTemp());
            if (data.getBoxId() == null) {
                sportsList.add(0);
            } else {
                sportsList.add(data.getGapSport());
            }
            electricList.add(data.getEartagPower());
            sleeptime.add(data.getSleepStatus());  //backfirst  用来存睡眠状态了
            environmenttempList.add(data.getEnvironmentTemp());
        }
        map1.put("time",timeList);
        map1.put("temp",tempList);
        map1.put("sports",sportsList);
        map1.put("electric",electricList);
        map1.put("environmenttemp",environmenttempList);
        map1.put("sleeptime",sleeptime);

        return map1;
    }



    //获取睡眠时间段的方法
    @Override
    public List<Map<String,String>> querySleepTimeBytimeAndEartag(Map<String, Object> params) throws ParseException {
        String eartag = (String) params.get("eartag");
        String starttime = (String) params.get("startTime");
        String endtime = (String) params.get("endTime");
        DateFormat fmt = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        //格式化传入的时间
        Date  startTime = fmt.parse(starttime);
        Date  endTime = fmt.parse(endtime);
        LambdaQueryWrapper<EartagDataEntity> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.eq(EartagDataEntity::getEartagId,eartag);
        lambdaQuery.eq(EartagDataEntity::getSleepGap,1);
        //过滤时间
        lambdaQuery.between(EartagDataEntity::getCreateTime,startTime, endTime);
        List<EartagDataEntity> eartagDataEntities = baseMapper.selectList(lambdaQuery);
        List<Map<String,String>> list = new ArrayList<>();
        for (EartagDataEntity eartagDataEntity : eartagDataEntities) {
                Map<String,String> map = new HashMap();
                map.put("starttime",fmt.format(eartagDataEntity.getCreateTime()));
                map.put("endtime",fmt.format(eartagDataEntity.getCreateTime()));
                map.put("status", String.valueOf(eartagDataEntity.getSleepGap()));
                map.put("timegap",eartagDataEntity.getRemark());
                list.add(map);
        }
        return list ;
    }

    @Override
    public Map getTemperatureCurve(Map<String, Object> params) {
        String eartag = (String)params.get("eartag");
        String startTime = (String)params.get("startTime");
        String endTime = (String)params.get("endTime");

        LambdaQueryWrapper<EartagDataEntity> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.eq(StringUtils.isNotBlank(eartag), EartagDataEntity::getEartagId,eartag);
        lambdaQuery.le(StringUtils.isNotBlank(endTime), EartagDataEntity::getCreateTime,endTime);
        lambdaQuery.ge(StringUtils.isNotBlank(startTime), EartagDataEntity::getCreateTime,startTime);

        List<EartagDataEntity> eartagDateEntities = baseMapper.selectList(lambdaQuery);

        List timeList = new ArrayList();
        List dataList = new ArrayList();
        for (EartagDataEntity eartagDateEntity : eartagDateEntities) {
            timeList.add(eartagDateEntity.getCreateTime());
            dataList.add(eartagDateEntity.getEartagTemp());
        }

        Map map = new HashMap();
        map.put("timeList",timeList);
        map.put("dataList",dataList);
        return map;
    }
}

