package com.huimv.management.timertask;

import cn.hutool.http.HttpUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.huimv.management.dao.FarmDao;
import com.huimv.management.dao.ThresholdManagementDao;
import com.huimv.management.entity.*;
import com.huimv.management.service.OutdoorEnvironmentService;
import com.huimv.management.service.ThresholdManagementService;
import com.huimv.management.service.WarningInfoService;
import com.huimv.management.service.impl.FarmServiceImpl;
import lombok.extern.slf4j.Slf4j;
import net.sf.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import java.nio.charset.Charset;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Author Anchor
 * @Date 2021/7/14 15:06
 * @Version 1.0.1
 *
 * 定时刷新获取天气的定时任务
 */

@Component
@SuppressWarnings("unused")
@Slf4j
public class WeatherTimerTask {

    //临时设置为1
    private static final int FARM_ID = 1;
    //注入阈值表
    @Autowired
    private ThresholdManagementService thresholdManagementService;

    @Autowired
    private ThresholdManagementDao thresholdManagementDao;

    @Autowired
    private WarningInfoService warningInfoService;

    @Autowired
    private FarmServiceImpl farmService;

    @Autowired
    private FarmDao farmDao;

    @Autowired
    private OutdoorEnvironmentService  outdoorEnvironmentService;


    //目前定义阈值在 mgt_threshold_management表   其中 belong_to =2  指外部环境
    //库中  id=3
    //gaowen:2   湿度2   空气质量
    //每天零点更新时间    //测试通过
    //@Scheduled(cron = "0 30 1 1 * ?")


    @Scheduled(cron = " 0 30 0 * * ? ")  //每天凌晨执行
    public  void getAndsetWeatherinfo() throws ParseException {

        List<FarmEntity> farmEntities = getfarmInfo();
        for (FarmEntity farmEntity : farmEntities) {
            log.info("当前查询"+farmEntity.getName());
            String[] location =  farmEntity.getFrontLocation().split(",");
            String province =  location[0];
            String city =  location[1];
            String county = location[2];
            int farmid =farmEntity.getId();
            getEnvironmentData(province,city,county,farmid);
        }
    }


    private void getEnvironmentData( String province ,  String city , String county,int farmid ) throws ParseException {

        log.info("当前城市为=======>" +province+city+county);
        //直接通过它计算出24小时之内的温蒂同时以一个新的数组存入表中，数据的取出和比较值后面进行
        String url = "https://wis.qq.com/weather/common";
        String param = "source=xw&weather_type=observe|forecast_1h|forecast_24h|index|alarm|limit|tips&province=" + province + "&city=" + city + "&county=" + county;
        String result = HttpUtil.urlWithForm(url, param, Charset.defaultCharset(), true);
        System.out.println(result);
        String s = HttpUtil.get(result);
        //对S进行解析，首先获取当天24小时的温度情况
        JSONObject jsonObject = JSONObject.fromObject(s);
        Object data = jsonObject.get("data");
        JSONObject jsonData = JSONObject.fromObject(data);

         Object forecastobject24hDetail = JSONObject.fromObject(jsonData.get("forecast_24h")).get("1"); //24小时天气预报的  “1”  为当天的天气预报

         Object maxDegree = JSONObject.fromObject(forecastobject24hDetail).get("max_degree");//当天最高温
         Object minDegree = JSONObject.fromObject(forecastobject24hDetail).get("min_degree");//当天最低温
         Object humidityValu =  JSONObject.fromObject(jsonData.get("observe")).get("humidity");//湿度
        log.info("湿度=======>" + humidityValu);
        log.info("最高温====>" + maxDegree);
        log.info("最低温====>" + minDegree);

        //空气质量：这个需要调用聚合天气的接口来实现
        String airQualityIndex = AirQualityIndexUtil.queryWeather( city.substring(0, city.length()-1));
        log.info("空气质量为====>" + airQualityIndex);

        JSONObject forecastByhour = JSONObject.fromObject(jsonData.get("forecast_1h"));  //获得逐小时数据并遍历

         //存储对应数据  零点执行，对应的前24条数据为当天的数据
        for (int i = 0; i <24; i++) {
            System.out.println("逐小时数据为" + forecastByhour.get(String.valueOf(i)));
            //现在需要将逐小时的气温数据存在表里面  //初步实现，裁剪
            JSONObject HoureDate = JSONObject.fromObject(forecastByhour.get(String.valueOf(i)));
             String timeString = (String)HoureDate.get("update_time");
             //时间为这种格式，进行格式化处理   20210714180000
             StringBuffer stringDate=new StringBuffer(timeString).insert(4,"-").insert(7,"-").insert(10," ").insert(13,":").insert(16,":");
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String TimeNow = sdf.format(Calendar.getInstance().getTime());
            Date date = sdf.parse(String.valueOf(stringDate));
            System.out.println(date);
            //存储第二天逐小时天气预报以及信息
            saveEnvironmentData(farmid, Double.parseDouble( (String)HoureDate.get("degree"))
                           ,Double.parseDouble(String.valueOf(humidityValu)),  Double.parseDouble(String.valueOf(airQualityIndex)),date);
        }
        //存储全天信息报警信息
        judgeWarningInfo(Double.parseDouble( String.valueOf(maxDegree)), Double.parseDouble(String.valueOf(minDegree)),
                Double.parseDouble(String.valueOf(airQualityIndex)), Double.parseDouble(String.valueOf(humidityValu)), farmid);
    }


    //判断环境的详细信息
    public void  saveEnvironmentData(  int farm_id ,Double temperature ,Double humidity ,Double air_quality ,Date date) {
        //true正常   false 异常

          Boolean abnormal_status =false;

        Map<String, Double> environmentThreadHold = getEnvironmentThreadHold(farm_id); //放的1  目前.传入牧场ID获取专属阈值

        //三元表达式，防止报警表没有对应的值

        Double sethighTemp = environmentThreadHold.get("环境温度高温")==null ?  WarmDataDefaultConstant.Default_High_Temp_Set :   environmentThreadHold.get("环境温度高温");
        Double setlowTemp = environmentThreadHold.get("环境温度低温")==null ? WarmDataDefaultConstant.Default_Low_Temp_Set :   environmentThreadHold.get("环境温度低温");
        Double setHighHumidityValu = environmentThreadHold.get("环境湿度高阈值")==null ?  WarmDataDefaultConstant.Default_High_humidityValu_set :   environmentThreadHold.get("环境湿度高阈值");
        Double setLowHumidityValu = environmentThreadHold.get("环境湿度低阈值")==null ?  WarmDataDefaultConstant.Default_Low_humidityValu_set:   environmentThreadHold.get("环境湿度低阈值");
        Double setHAirqualityindex = environmentThreadHold.get("空气质量不达标阈值")==null ? WarmDataDefaultConstant.Default_airqualityindex :   environmentThreadHold.get("空气质量不达标阈值");

        //1.全部合格设置为true
        if (sethighTemp > temperature ||setlowTemp  < temperature || setHighHumidityValu  > humidity || setLowHumidityValu < humidity ||  setHAirqualityindex > air_quality) {
              abnormal_status =true;
        }
        OutdoorEnvironmentEntity outdoorEnvironmentEntity = new OutdoorEnvironmentEntity();
        outdoorEnvironmentEntity.setAbnormalStatus(abnormal_status);
        outdoorEnvironmentEntity.setAirQuality(String.valueOf(air_quality));
        outdoorEnvironmentEntity.setFarmId(farm_id);
        outdoorEnvironmentEntity.setTemperature(temperature);
        outdoorEnvironmentEntity.setHumidity(humidity);
        outdoorEnvironmentEntity.setDate(date );
        outdoorEnvironmentEntity.setCollectTime(new Date());
        outdoorEnvironmentEntity.setCollectTime(date);

        outdoorEnvironmentService.save(outdoorEnvironmentEntity);
    }





    //对-环境高温--环境低温--环境湿度过高--环境湿度过低--空气质量过差--进行比较，确定是否报警，并存在表里面
    public void judgeWarningInfo(Double maxtemp, Double mintemp, Double airqualityindex, Double humidityValu, int farmId) throws ParseException {

        Map<String, Double> environmentThreadHold = getEnvironmentThreadHold(farmId); //放的1  目前.传入牧场ID获取专属阈值

          //三元表达式，防止报警表没有对应的值
          Double sethighTemp = environmentThreadHold.get("环境温度高温")==null ?  WarmDataDefaultConstant.Default_High_Temp_Set :   environmentThreadHold.get("环境温度高温");
          Double setlowTemp = environmentThreadHold.get("环境温度低温")==null ? WarmDataDefaultConstant.Default_Low_Temp_Set :   environmentThreadHold.get("环境温度低温");
          Double setHighHumidityValu = environmentThreadHold.get("环境湿度高阈值")==null ?  WarmDataDefaultConstant.Default_High_humidityValu_set :   environmentThreadHold.get("环境湿度高阈值");
          Double setLowHumidityValu = environmentThreadHold.get("环境湿度低阈值")==null ?  WarmDataDefaultConstant.Default_Low_humidityValu_set:   environmentThreadHold.get("环境湿度低阈值");
          Double setHAirqualityindex = environmentThreadHold.get("空气质量不达标阈值")==null ? WarmDataDefaultConstant.Default_airqualityindex :   environmentThreadHold.get("空气质量不达标阈值");

        //1.高温
        if(sethighTemp>maxtemp){
            addwarningInfo(farmId, "环境高温报警",maxtemp);
        }
        //1.低温
        if(setlowTemp>maxtemp){
            addwarningInfo(farmId, "环境低温报警",mintemp);
        }
        //1.湿度过高
        if(setHighHumidityValu>humidityValu){
            addwarningInfo(farmId, "环境湿度过高报警",humidityValu);
        }
        //1.湿度过低
        if(setLowHumidityValu>humidityValu){
            addwarningInfo(farmId, "环境湿度过低报警",humidityValu);
        }
        //1.空气质量不达标
        if(setHAirqualityindex< airqualityindex){
            addwarningInfo(farmId, "空气质量不达标报警",airqualityindex);
        }
    }





    //获取阈值   牧场ID是有用的，需要判断是哪个牧场的数据
    public Map<String, Double> getEnvironmentThreadHold(int farmId) {
        //ThresholdManagementEntity thresholdManagementEntity
        LambdaQueryWrapper<ThresholdManagementEntity> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(ThresholdManagementEntity::getDeleted, 0);//未删除的阈值
        wrapper.eq(ThresholdManagementEntity::getBelongTo, 2);//室外的阈值
        wrapper.eq(ThresholdManagementEntity::getFarmId, farmId);//室外的阈值
        List<ThresholdManagementEntity> thresholdManagementEntityList = thresholdManagementDao.selectList(wrapper);


        Map<String, Double> thresholdManagementEntityMap = new HashMap<>();
        for (ThresholdManagementEntity c : thresholdManagementEntityList) {
            thresholdManagementEntityMap.put(c.getThresholdType(), c.getThresholdValue());
        }
        return thresholdManagementEntityMap;
    }


    //存报警信息
    public void addwarningInfo(int farmId, String warningType, Double now_value) throws ParseException {

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd ");
        String TimeNow = sdf.format(Calendar.getInstance().getTime());
        Date formatTimeNow = sdf.parse(TimeNow);

        WarningInfoEntity warningInfoEntity = new WarningInfoEntity();
        warningInfoEntity.setBelongTo((short) 2);
        warningInfoEntity.setFarmId(farmId);
        warningInfoEntity.setWarningType(warningType); //例如：环境高温报警
        warningInfoEntity.setNowDate(formatTimeNow);//设置时间 2021-07-08  此格式
        warningInfoEntity.setNowValue(now_value);
        warningInfoEntity.setWarningTime(new Date());//设置时间 2021-06-25 14:10:54 此格式
        warningInfoService.save(warningInfoEntity);

    }

    //获取所有未被删除的牧场列表
    public List<FarmEntity> getfarmInfo() throws ParseException {

        LambdaQueryWrapper<FarmEntity> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(FarmEntity::getDeleted, 0);//未删除的阈值这个字段为O
        List<FarmEntity> FarmEntityList = farmDao.selectList(wrapper);

        return FarmEntityList;
    }
}